(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

(* $Id: syntaxerr.ml,v 1.3 2000/03/16 07:58:16 judaigi Exp $ *)

(* Auxiliary type for reporting syntax errors *)

open Format
open Location

type err_kinds = IllegalAsPat | IllegalLabel | IllegalField | IllegalPattern
               | IllegalRecBind | UnknownError | IllegalExpSeq

type errmsg_kinds = EngGentle | HanGentle | HanTough

let errmsg_type = ref HanGentle

type error =
    Unclosed of Location.t * string * Location.t * string
  | Duptyid of Location.t * Location.t * string
  | Dupdatacon of Location.t * Location.t * string
  | Duppatvar of Location.t * Location.t * string
  | Orpaterr of Location.t
  | Other of Location.t * err_kinds

exception Error of error
exception Escape_error

let print_err_kinds e =
  match e with
    IllegalAsPat -> print_string 
      (match !errmsg_type with
         EngGentle -> "As pattern must be used with variable pattern."
       | HanGentle -> "As Ͽ ׻  ϰ  մϴ."
       | HanTough  -> "As տ ׻  ´ٴϱ!!")
  | IllegalLabel -> print_string 
      (match !errmsg_type with
         EngGentle -> "Illegal record label."
       | HanGentle -> "߸ ڵ ǥ(label)Դϴ."
       | HanTough  -> "̻Ѱ ڲ ڵ ǥ , ڸ̴ "^
                      "ڸ̴ ؾ Ѵٴϱ!!")
  | IllegalField -> print_string 
      (match !errmsg_type with
         EngGentle -> "Illegal field of record pattern."
       | HanGentle -> "ڵ ʵ尡 ̻մϴ."
       | HanTough  -> "ڵ ȿٰ ̻Ѱ ??")
  | IllegalPattern -> print_string 
      (match !errmsg_type with
         EngGentle -> "Illegal pattern."
       | HanGentle -> "߸ Դϴ."
       | HanTough  -> "̰͵ ̶ ??")
  | IllegalRecBind -> print_string 
      (match !errmsg_type with
         EngGentle -> "Recursive variable must be bound to 'fn' expression."
       | HanGentle -> " ǵ  ׻ 'fn' ǥ ; մϴ."
       | HanTough  -> "let rec  ҷ, 'fn' Ŷ, ٸ  "^
       		      "   𸣴ϱ.")
  | IllegalExpSeq -> print_string 
      (match !errmsg_type with
         EngGentle -> "Sequence of expression must end with expression"^
	 	      "(Not semi-colon)."
       | HanGentle -> "α׷ ǥ  ׻ α׷ ǥ  "^
       		      "մϴ(ݷ ȵ˴ϴ)."
       | HanTough  -> "α׷ ǥ ҷ   α׷ "^
       		      "ǥ̾ Ѵٱ. ݷ  ɷ  ȵ.")
  | UnknownError -> print_string 
      (match !errmsg_type with
         EngGentle -> "Unknown error ocurred."
       | HanGentle -> "ϴ  ߻߽ϴ."
       | HanTough  -> "̰  𸣰ڴ.  ˾Ƽ ˾ƺ.")
  

let report_error = function
    Unclosed(opening_loc, opening, closing_loc, closing) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations opening_loc closing_loc
      then begin
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: '"
	   | HanGentle -> " : '"
	   | HanTough  -> "û Ǽ: '");
        print_string closing;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' expected, the highlighted '"
	   | HanGentle -> "' ()  ֽϴ. ǥõ '"
	   | HanTough  -> "'  Գ!  ǥ '");
        print_string opening;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' might be unmatched"
	   | HanGentle -> "' () ¦  ʴ  ϴ."
	   | HanTough  -> "'  ȸ ̴. Ķ.");
      end else begin
        Location.print closing_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: '"
	   | HanGentle -> " : '"
	   | HanTough  -> "û Ǽ: '");
        print_string closing;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' expected "
	   | HanGentle -> "' ()  ֽϴ. "
	   | HanTough  -> "'  Ծ! "); 
	force_newline();
        Location.print opening_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "This '"
	   | HanGentle -> " ġ '"
	   | HanTough  -> " ǥ '");
        print_string opening;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' might be unmatched"
	   | HanGentle -> "' () ¦  ʴ  ϴ."
	   | HanTough  -> "'  ȸ ̴. Ķ.")
      end
  | Duptyid(previous_loc, current_loc, name) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then begin
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: type name '"
	   | HanGentle -> " : Ÿ ̸ '"
	   | HanTough  -> "û Ǽ: Ÿ ̸ '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' is duplicated at the highlighted positions"
	   | HanGentle -> "' () ϳ Ÿǳ ι Ͽϴ. "
			  ^"ǥõ κ   ǵ κ Դϴ."
	   | HanTough  -> "'  ǥ κп Ȱ  . ׷ ȵǿ~")
      end else begin
        Location.print current_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: type name '"
	   | HanGentle -> " : Ÿ ̸ '"
	   | HanTough  -> "û Ǽ: Ÿ ̸ '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' previously appeared."; 
	   | HanGentle -> "'  ̹ ϳ Ÿǳ ̹ ǵǾϴ."
	   | HanTough  -> "'  ̹ տ ֱ. ׷ ȵǿ~~");
	force_newline();
        Location.print previous_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Previous position of the type name '";
	   | HanGentle -> " ġ ̹ ǵǾ ֽϴ."
	   | HanTough  -> "   ,  ϳ ٲ.");
	if !errmsg_type = EngGentle
	then ( print_string name; print_string "'" )
	else ()
      end
  | Duppatvar(previous_loc, current_loc, name) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then begin
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: pattern variable '"
	   | HanGentle -> " :   '"
	   | HanTough  -> "û Ǽ:   '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' is duplicated at the highlighted positions"
	   | HanGentle -> "' ()  Ͽ ι ϴ. "^
	   		  "ǥõ  Ȯϼ"
	   | HanTough  -> "'   Ͽ ι    ϶? "^
			  " ǥ  Ȯ .")
      end else begin
        Location.print current_loc;
	print_string
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: pattern variable '"
	   | HanGentle -> " :   '"
	   | HanTough  -> "û Ǽ:   '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' previously appeared."
	   | HanGentle -> "' ()  ϳ ̹ ϴ."
	   | HanTough  -> "'  ̹  ִ ̱. ");
	force_newline();
	Location.print previous_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Previous position of the pattern variable '"
	   | HanGentle -> " ġ ̹ ǵǾ ֽϴ."
	   | HanTough  -> "   ,  ϳ ٲ.");
	if !errmsg_type = EngGentle
	then ( print_string name; print_string "'" )
	else ()
      end
  | Dupdatacon(previous_loc, current_loc, name) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations previous_loc current_loc
      then begin
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: data constructor '"
	   | HanGentle -> " : Ÿ  '"
	   | HanTough  -> "û Ǽ: Ÿ  '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' is duplicated at the highlighted positions."
	   | HanGentle -> "'   ǳ  ̻ ϴ."
	   | HanTough  -> "'  ι    ϳ? ϳ ٲ.")
      end else begin
        Location.print current_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: data constructor '"
	   | HanGentle -> " : Ÿ  '"
	   | HanTough  -> "û Ǽ: Ÿ  '");
        print_string name;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "' previously appeared."
	   | HanGentle -> "'   ǳ ̹ ̰ ֽϴ."
	   | HanTough  -> "'  ̹ ǵ ε? -.-");
	force_newline();
        Location.print previous_loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Previous position of the data constructor '"
	   | HanGentle -> " ġ ̹ ǵǾ ֽϴ."
	   | HanTough  -> "   ,  ϳ ٲ.");
        print_string name;
	print_string "'"
      end
  | Orpaterr (loc) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations loc loc
      then begin
      print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: Or patterns must have same "^
	     		  "pattern variables"
	   | HanGentle -> " : Or   ϵ  "^
	   		  " ƾ մϴ."
	   | HanTough  -> "û Ǽ: Or ϵ鿡    "^
	   		  "ƾ Ѵٱ. ȱ׷ ̻ݾ?");
      end else begin
      Location.print loc;
      print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: Or patterns must have same "^
	     		  "pattern variables"
	   | HanGentle -> " : Or   ϵ  "^
	   		  " ƾ մϴ."
	   | HanTough  -> "û Ǽ: Or ϵ鿡    "^
	   		  "ƾ Ѵٱ. ȱ׷ ̻ݾ?");
      end
  | Other (loc,e) ->
      if String.length !Location.input_name = 0
      && Location.highlight_locations loc loc
      then begin
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: "
	   | HanGentle -> " : "
	   | HanTough  -> "û Ǽ: ");
        print_err_kinds e
      end else begin
        Location.print loc;
        print_string 
	  (match !errmsg_type with
	     EngGentle -> "Syntax error: "
	   | HanGentle -> " : "
	   | HanTough  -> "û Ǽ: ");
        print_err_kinds e
      end

